/**
 * Copyright (c) 2017 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Notwithstanding the foregoing, you may not use, copy, modify, merge, publish,
 * distribute, sublicense, create a derivative work, and/or sell copies of the
 * Software in any work that is designed, intended, or marketed for pedagogical or
 * instructional purposes related to programming, coding, application development,
 * or information technology.  Permission for such use, copying, modification,
 * merger, publication, distribution, sublicensing, creation of derivative works,
 * or sale is expressly withheld.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import Cocoa

class ViewController: NSViewController {
  
  @IBOutlet weak var tableView: NSTableView!
  @IBOutlet weak var imageView: NSImageView!
  @IBOutlet weak var nameField: NSTextField!
  @IBOutlet weak var ratingIndicator: NSLevelIndicator!
  @IBOutlet var noteView: NSTextView!
  @IBOutlet weak var beerCountField: NSTextField!
  
  var selectedBeer: Beer? {
    didSet {
      guard let selectedBeer = selectedBeer else {
        setFieldsEnabled(enabled: false)
        imageView.image = nil
        nameField.stringValue = ""
        ratingIndicator.integerValue = 0
        noteView.string = ""
        return
      }
      setFieldsEnabled(enabled: true)
      imageView.image = selectedBeer.beerImage()
      nameField.stringValue = selectedBeer.name
      ratingIndicator.integerValue = selectedBeer.rating
      noteView.string = selectedBeer.note!
    }
  }
  
  override func viewDidLoad() {
    super.viewDidLoad()
    if BeerManager.sharedInstance.beers.count == 0 {
      setFieldsEnabled(enabled: false)
    } else {
      tableView.selectRowIndexes(IndexSet(integer: 0), byExtendingSelection: false)
    }
    updateBeerCountLabel()
  }
  
  private func setFieldsEnabled(enabled: Bool) {
    imageView.isEditable = enabled
    nameField.isEnabled = enabled
    ratingIndicator.isEnabled = enabled
    noteView.isEditable = enabled
  }
  
  private func updateBeerCountLabel() {
    beerCountField.stringValue = "\(BeerManager.sharedInstance.beers.count)"
  }
  
  @IBAction func selectImage(_ sender: NSClickGestureRecognizer) {
    guard let window = view.window else { return }
    // 1.
    let openPanel = NSOpenPanel()
    openPanel.allowsMultipleSelection = false
    openPanel.canChooseDirectories = false
    openPanel.canCreateDirectories = false
    openPanel.canChooseFiles = true
    
    // 2.
    openPanel.allowedFileTypes = ["jpg", "png", "tiff"]
    
    // 3.
    openPanel.beginSheetModal(for: window) { (result) in
      if result == NSApplication.ModalResponse.OK {
        // 4.
        if let panelURL = openPanel.url,
          let beerImage = NSImage(contentsOf: panelURL) {
          self.selectedBeer?.saveImage(beerImage)
          self.imageView.image = beerImage
        }
      }
    }
  }
  
  @IBAction func imageChanged(_ sender: NSImageView) {
    guard let image = sender.image else { return }
    selectedBeer?.saveImage(image)
  }
  
  @IBAction func updateBeer(_ sender: NSButton) {
    // 1.
    guard let beer = selectedBeer,
      let index = BeerManager.sharedInstance.beers.index(of: beer) else { return }
    beer.name = nameField.stringValue
    beer.rating = ratingIndicator.integerValue
    beer.note = noteView.string
    
    // 2.
    let indexSet = IndexSet(integer: index)
    tableView.beginUpdates()
    tableView.reloadData(forRowIndexes: indexSet, columnIndexes: IndexSet(integer: 0))
    tableView.endUpdates()
    
    // 3.
    BeerManager.sharedInstance.saveBeers()
  }
  
  @IBAction func addBeer(_ sender: NSButton) {
    // 1.
    let beer = Beer()
    beer.name = ""
    beer.rating = 1
    beer.note = ""
    selectedBeer = beer
    
    // 2.
    BeerManager.sharedInstance.beers.insert(beer, at: 0)
    BeerManager.sharedInstance.saveBeers()
    
    // 3.
    let indexSet = IndexSet(integer: 0)
    tableView.beginUpdates()
    tableView.insertRows(at: indexSet, withAnimation: .slideDown)
    tableView.endUpdates()
    updateBeerCountLabel()
    
    // 4.
    tableView.selectRowIndexes(IndexSet(integer: 0), byExtendingSelection: false)
  }
  
  @IBAction func removeBeer(_ sender: NSButton) {
    guard let beer = selectedBeer,
      let index = BeerManager.sharedInstance.beers.index(of: beer) else {
        return
    }
    
    // 1.
    BeerManager.sharedInstance.beers.remove(at: index)
    BeerManager.sharedInstance.saveBeers()
    
    // 2
    tableView.reloadData()
    updateBeerCountLabel()
    tableView.selectRowIndexes(IndexSet(integer: 0), byExtendingSelection: false)
    if BeerManager.sharedInstance.beers.count == 0 {
      selectedBeer = nil
    }
  }
  
}

extension ViewController: NSTableViewDataSource {
  func numberOfRows(in tableView: NSTableView) -> Int {
    return BeerManager.sharedInstance.beers.count
  }
}

extension ViewController: NSTableViewDelegate {
  // MARK: - CellIdentifiers
  fileprivate enum CellIdentifier {
    static let NameCell = "NameCell"
  }
  
  func tableView(_ tableView: NSTableView, viewFor tableColumn: NSTableColumn?, row: Int) -> NSView? {
    
    let beer = BeerManager.sharedInstance.beers[row]
    
    if let cell = tableView.makeView(withIdentifier: NSUserInterfaceItemIdentifier(rawValue: CellIdentifier.NameCell), owner: nil) as? NSTableCellView {
      cell.textField?.stringValue = beer.name
      if beer.name.characters.count == 0 {
        cell.textField?.stringValue = "New Beer"
      }
      return cell
    }
    return nil
  }
  
  func tableViewSelectionDidChange(_ notification: Notification) {
    if tableView.selectedRow >= 0 {
      selectedBeer = BeerManager.sharedInstance.beers[tableView.selectedRow]
    }
  }
}

extension ViewController: NSTextViewDelegate {
  func textView(_ textView: NSTextView, shouldChangeTextIn affectedCharRange: NSRange, replacementString: String?) -> Bool {
    guard let replacementString = replacementString else { return true }
    let currentText = textView.string
    let proposed = (currentText as NSString).replacingCharacters(in: affectedCharRange, with: replacementString)
    return proposed.isValidLength()
  }
}
